<?php

declare(strict_types=1);

class ControllerExtensionPaymentConnectPay extends Controller
{
  private array $error = [];
  private array $paymentCountries = ['DE', 'FI', 'NL', 'LT'];
  private const REQUEST_URL = 'https://api2.connectpay.com/';

  public function index(): void
  {
    $this->load->language('extension/payment/connectpay');
    $this->document->setTitle($this->language->get('heading_title'));
    $this->document->addStyle('view/stylesheet/connectpay/connectpay.css');
    $this->document->addScript('view/javascript/connectpay/connectpay.js');
    $this->load->model('setting/setting');
    $this->load->model('extension/payment/connectpay');
    $this->load->model('localisation/order_status');

    if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
      $this->request->post['payment_connectpay_private_key'] = $this->getEncodedPrivateKey();
      $this->request->post['payment_connectpay_all_countries'] = 0;

      if ($this->request->post['payment_connectpay_payment_country'] === $this->language->get('entry_all_available_countries')) {
        $this->request->post['payment_connectpay_all_countries'] = 1;
      }

      $this->model_setting_setting->editSetting('payment_connectpay', $this->request->post);
      $data['success'] = $this->language->get('text_success');
    }

    if (isset($this->error['warning'])) {
      $data['error_warning'] = $this->error['warning'];
    } else {
      $data['error_warning'] = '';
    }

    if (isset($this->error['cp_client_id'])) {
      $data['error_cp_client_id'] = $this->error['cp_client_id'];
    } else {
      $data['error_cp_client_id'] = '';
    }

    if (isset($this->error['cp_client_secret'])) {
      $data['error_cp_client_secret'] = $this->error['cp_client_secret'];
    } else {
      $data['error_cp_client_secret'] = '';
    }

    if (isset($this->error['cp_brand_id'])) {
      $data['error_cp_brand_id'] = $this->error['cp_brand_id'];
    } else {
      $data['error_cp_brand_id'] = '';
    }

    if (isset($this->error['cp_payment_method'])) {
      $data['error_cp_payment_method'] = $this->error['cp_payment_method'];
    } else {
      $data['error_cp_payment_method'] = '';
    }

    if (isset($this->error['cp_private_key'])) {
      $data['error_cp_private_key'] = $this->error['cp_private_key'];
    } else {
      $data['error_cp_private_key'] = '';
    }

    if (isset($this->error['cp_payment_method_not_selected'])) {
      $data['error_cp_payment_method_not_selected'] = $this->error['cp_payment_method_not_selected'];
    } else {
      $data['error_cp_payment_method_not_selected'] = '';
    }

    if (isset($this->error['cp_payment_country_not_selected'])) {
      $data['error_cp_payment_country_not_selected'] = $this->error['cp_payment_country_not_selected'];
    } else {
      $data['error_cp_payment_country_not_selected'] = '';
    }

    $data['breadcrumbs'] = [];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('text_home'),
      'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
    ];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('text_extension'),
      'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
    ];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('heading_title'),
      'href' => $this->url->link('extension/payment/connectpay', 'user_token=' . $this->session->data['user_token'], true)
    ];

    $data['action'] = $this->url->link('extension/payment/connectpay', 'user_token=' . $this->session->data['user_token'], true);

    $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);

    if (isset($this->request->post['payment_connectpay_client_id'])) {
      $data['payment_connectpay_client_id'] = $this->request->post['payment_connectpay_client_id'];
    } else {
      $data['payment_connectpay_client_id'] = $this->config->get('payment_connectpay_client_id');
    }

    if (isset($this->request->post['payment_connectpay_client_secret'])) {
      $data['payment_connectpay_client_secret'] = $this->request->post['payment_connectpay_client_secret'];
    } else {
      $data['payment_connectpay_client_secret'] = $this->config->get('payment_connectpay_client_secret');
    }

    if (isset($this->request->post['payment_connectpay_brand_id'])) {
      $data['payment_connectpay_brand_id'] = $this->request->post['payment_connectpay_brand_id'];
    } else {
      $data['payment_connectpay_brand_id'] = $this->config->get('payment_connectpay_brand_id');
    }

    if (isset($this->request->post['payment_connectpay_private_key'])) {
      $data['payment_connectpay_private_key'] = $this->request->post['payment_connectpay_private_key'];
    } else {
      $data['payment_connectpay_private_key'] = $this->getEncodedPrivateKey();
    }

    if (isset($this->request->post['payment_connectpay_payment_method'])) {
      $data['payment_connectpay_payment_method'] = $this->request->post['payment_connectpay_payment_method'];
    } else {
      $data['payment_connectpay_payment_method'] = $this->config->get('payment_connectpay_payment_method');
    }

    $data['payment_connectpay_payment_providers'] = [];

    if ($this->config->get('payment_connectpay_payment_country') && $this->config->get('payment_connectpay_private_key')) {
      $data['payment_connectpay_payment_providers'] = json_decode($this->getPaymentProviders());
    }

    $data['all_available_countries_list'] = $this->paymentCountries;
    $data['orderStatuses'] = $this->model_localisation_order_status->getOrderStatuses();

    if (isset($this->request->post['payment_connectpay_payment_providers'])) {
      $data['payment_connectpay_selected_payment_providers'] = $this->request->post['payment_connectpay_payment_providers'];
    } else {
      $data['payment_connectpay_selected_payment_providers'] = $this->config->get('payment_connectpay_payment_providers');
    }
    $data['payment_connectpay_payment_countries'] = $this->getPaymentCountries();

    if (isset($this->request->post['payment_connectpay_payment_country'])) {
      $data['payment_connectpay_selected_payment_country'] = $this->request->post['payment_connectpay_payment_country'];
    } else {
      $data['payment_connectpay_selected_payment_country'] = $this->config->get('payment_connectpay_payment_country');
    }

    if (isset($this->request->post['payment_connectpay_status'])) {
      $data['payment_connectpay_status'] = $this->request->post['payment_connectpay_status'];
    } else {
      $data['payment_connectpay_status'] = $this->config->get('payment_connectpay_status');
    }

    $data['header'] = $this->load->controller('common/header');
    $data['column_left'] = $this->load->controller('common/column_left');
    $data['footer'] = $this->load->controller('common/footer');
    $data['get_payment_providers'] = $this->url->link('extension/payment/connectpay/getPaymentProviders', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
    $data['all_countries_translation'] = $this->language->get('entry_all_available_countries');
    $data['cp_payment_method_not_selected_disclaimer'] = $this->language->get('error_cp_payment_method_not_selected');
    $data['cp_payment_provider_not_selected_disclaimer'] = $this->language->get('error_cp_payment_provider_not_selected');

    $storeUrl = HTTPS_CATALOG ?? HTTP_CATALOG;
    $data['payment_connectpay_cron_url'] = $storeUrl . 'index.php?route=extension/payment/connectpay/cron';

    if (isset($this->request->post['payment_connectpay_order_status_pending'])) {
      $data['payment_connectpay_order_status_pending'] = $this->request->post['payment_connectpay_order_status_pending'];
    } else {
      $data['payment_connectpay_order_status_pending'] = $this->config->get('payment_connectpay_order_status_pending');
    }

    if (isset($this->request->post['payment_connectpay_order_status_approved_settled'])) {
      $data['payment_connectpay_order_status_approved_settled'] = $this->request->post['payment_connectpay_order_status_approved_settled'];
    } else {
      $data['payment_connectpay_order_status_approved_settled'] = $this->config->get('payment_connectpay_order_status_approved_settled');
    }

    if (isset($this->request->post['payment_connectpay_order_status_failed_abandoned'])) {
      $data['payment_connectpay_order_status_failed_abandoned'] = $this->request->post['payment_connectpay_order_status_failed_abandoned'];
    } else {
      $data['payment_connectpay_order_status_failed_abandoned'] = $this->config->get('payment_connectpay_order_status_failed_abandoned');
    }

    if (isset($this->request->post['payment_connectpay_default_payment_country'])) {
      $data['payment_connectpay_default_payment_country'] = $this->request->post['payment_connectpay_default_payment_country'];
    } else {
      $data['payment_connectpay_default_payment_country'] = $this->config->get('payment_connectpay_default_payment_country');
    }

    if (isset($this->error['cp_incorrect_credentials'])) {
      $data['error_cp_incorrect_credentials'] = $this->error['cp_incorrect_credentials'];
    } else {
      $data['error_cp_incorrect_credentials'] = '';
    }

    $this->response->setOutput($this->load->view('extension/payment/connectpay', $data));
  }

  public function install() {
    if ($this->user->hasPermission('modify', 'marketplace/extension')) {
      $this->load->model('extension/payment/connectpay');

      $this->model_extension_payment_connectpay->install();
    }
  }

  public function uninstall() {
    if ($this->user->hasPermission('modify', 'marketplace/extension')) {
      $this->load->model('extension/payment/connectpay');

      $this->model_extension_payment_connectpay->uninstall();
    }
  }

  public function getPaymentProviders(): string
  {
    $countryCode = $this->config->get('payment_connectpay_payment_country');
    if ($countryCode !== 'DE') {
      $reqMethod = 'get';
      $pageNo = '1';
      $pageSize = '10000';
      $requestUrl = self::REQUEST_URL . "merchant/providers?countryCode=$countryCode&pageNo=$pageNo&pageSize=$pageSize";

      // Data to sign signature
      $dataToSign = $this->getDataToSign($requestUrl, $reqMethod);

      // Request headers
      $headers = [
        'Authorization: ' . $this->getAuthToken(),
        'X-Request-ID: ' . $this->getGuid(),
        'Accept: ' . 'application/json;version=2',
        'X-Signature: ' . $this->getSignature($dataToSign)
      ];

      // Response from GET Payment Providers request
      $curl = curl_init();
      curl_setopt_array($curl, [
        CURLOPT_URL => $requestUrl,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => '',
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_FOLLOWLOCATION => true,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => 'GET',
        CURLOPT_HTTPHEADER => $headers
      ]);

      curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
      $response = curl_exec($curl);
      curl_close($curl);

      if (false === $response) {
        return '';
      }

      if (curl_getinfo($curl, CURLINFO_HTTP_CODE) !== 200) {
        $responseBody = json_decode($response, true);
        foreach ($responseBody['messages'] ?? [] as $message) {
          $this->error['cp_incorrect_credentials'] = $message['message'];
          $this->log->write($message);
        }
      }
      return $response;
    }

    return '';
  }

  protected function validate(): bool
  {
    if (!$this->user->hasPermission('modify', 'extension/payment/connectpay')) {
      $this->error['warning'] = $this->language->get('error_connectpay_permission');
    }

    if (!$this->request->post['payment_connectpay_client_id']) {
      $this->error['cp_client_id'] = $this->language->get('error_cp_client_id');
    }

    if (!$this->request->post['payment_connectpay_client_secret']) {
      $this->error['cp_client_secret'] = $this->language->get('error_cp_client_secret');
    }

    if (!$this->request->post['payment_connectpay_brand_id']) {
      $this->error['cp_brand_id'] = $this->language->get('error_cp_brand_id');
    }

    if (null === $this->getEncodedPrivateKey()) {
      $this->error['cp_private_key'] = $this->language->get('error_cp_private_key');
    }

    if (!$this->request->post['payment_connectpay_payment_method']
      || ($this->request->post['payment_connectpay_payment_method']['PIS'] === '0'
        && $this->request->post['payment_connectpay_payment_method']['CARD'] === '0')
    ) {
      $this->error['cp_payment_method'] = $this->language->get('error_cp_payment_method');
    }

    if (!isset($this->request->post['payment_connectpay_payment_country'])) {
      $this->error['cp_payment_country_not_selected'] = $this->language->get('error_cp_payment_country_not_selected');
    }

    return !$this->error;
  }

  private function getPaymentCountries(): array
  {
    $this->paymentCountries[] = $this->language->get('entry_all_available_countries');
    return $this->paymentCountries;
  }

  private function getEncodedPrivateKey(): ?string
  {
    $certificateFormats = [
      'p8', 'key', 'p10', 'csr', 'crl',
      'pem', 'crt', 'p7b', 'spc', 'p7r',
      'p7s', 'der', 'cer', 'pfx', 'p12',
    ];
    if (isset($this->request->files["payment_connectpay_private_key"]["tmp_name"])
      && !empty($file = $this->request->files["payment_connectpay_private_key"]["tmp_name"])) {
      $fileExtension = explode('.', $this->request->files["payment_connectpay_private_key"]["name"]);

      if (!in_array($fileExtension[1], $certificateFormats)) {
        return null;
      } else {
        return base64_encode(file_get_contents($file));
      }
    }

    return $this->config->get('payment_connectpay_private_key');
  }

  private function getAuthToken(): string
  {
    $clientId = $this->config->get('payment_connectpay_client_id');
    $clientSecret = $this->config->get('payment_connectpay_client_secret');

    return 'Basic ' . base64_encode($clientId . ':' . $clientSecret);
  }

  private function getGuid(): string
  {
    mt_srand((int)microtime() * 10000);
    $charid = md5(uniqid((string)rand(), true));
    $hyphen = chr(45);

    return chr(123)
      . substr($charid, 0, 8) . $hyphen
      . substr($charid, 8, 4) . $hyphen
      . substr($charid, 12, 4) . $hyphen
      . substr($charid, 16, 4) . $hyphen
      . substr($charid, 20, 12)
      . chr(125);
  }

  private function getSignature(string $data): string
  {
    $privateKey = base64_decode($this->config->get('payment_connectpay_private_key'));
    $signature = '';
    $alg = 'sha256WithRSAEncryption';
    openssl_sign($data, $signature, $privateKey, $alg);

    return base64_encode($signature);
  }

  private function getDataToSign(string $requestUrl, string $reqMethod): string
  {
    $requestUrlForSignature = strtolower($requestUrl);
    $requestMethod = strtolower($reqMethod);
    $requestBody = strtolower('');

    // Get URL without https:// part
    $requestUrlForSignature = explode('//', $requestUrlForSignature)[1];

    // Data to sign signature
    return $requestMethod . '|' . $requestUrlForSignature . '|' . $requestBody;
  }
}
